import { createPropWithLangs, createPropsWithLangs } from '../../utils/form';

export default {
    props: {
        id: [Number, String],
    },

    data() {
        return {
            lang: 'uk',
            ready: false,
            updateCollection: {},
        };
    },

    computed: {
        langs() {
            return this.$settings.get('i18n.langs') || this.$store.state.langs.map((lang) => lang.code);
        },
    },

    async created() {
        if (this.id) {
            await this.fetch();
        } else {
            this.ready = true;
        }

        if (this.schema && this.schema.update) {
            for (const prop of this.schema.update) {
                this.$watch(`entity.${prop}`, (value) => {
                    this.updateCollection[prop] = value;
                });
            }
        }
    },

    methods: {
        validateField(type, name, value) {
            const errors = [];
            if (type === 'image') {
                if (!value || !value.url) {
                    errors.push(`${name}: Need to load image`);
                }
            }

            return errors;
        },

        createPropWithLangs(prop, defaultValue) {
            return createPropWithLangs(prop, this.langs, defaultValue);
        },

        createPropsWithLangs(props) {
            return createPropsWithLangs(props, this.langs);
        },

        async fetch() {
            try {
                this.entity = await this.$api[this.entityType].withId(this.id).get({
                    ml: true,
                });

                if (typeof this.afterFetch === 'function') {
                    await this.afterFetch();
                }

                this.ready = true;
            } catch (e) {
                console.error(e);
                this.$message.error({
                    message: this.$t('adminuikit.get.item.error.notify', { id: this.id }),
                });
            }
        },

        async update(input) {
            try {
                let data = {};
                if (this.schema && this.schema.update && Object.keys(this.updateCollection).length > 0) {
                    data = { ...this.updateCollection };
                } else {
                    data = input;
                }

                const entity = await this.$api[this.entityType]
                    .withId(this.entity.id)
                    .update(data, { lang: this.lang });

                this.$emit('load', entity);

                return entity;
            } catch (e) {
                throw e;
            }
        },

        async create(data) {
            try {
                const entity = await this.$api[this.entityType].create(data, { lang: this.lang });
                if (!entity || !entity.id) {
                    throw new Error('Entity is not created');
                }

                this.$emit('load', entity);

                return entity;
            } catch (e) {
                throw e;
            }
        },

        async save() {
            if (typeof this.beforeSave === 'function') {
                this.beforeSave();
            }

            if (typeof this.validate === 'function') {
                const valid = await this.validate();
                if (!valid) {
                    return;
                }
            }

            let data;
            if (typeof this.getSavingData === 'function') {
                data = this.getSavingData();
            } else {
                data = this.entity;
            }

            try {
                let entity = {};
                if (this.entity.id) {
                    entity = await this.update(data);
                } else {
                    entity = await this.create(data);
                }

                if (typeof this.afterSave === 'function') {
                    await this.afterSave(entity);
                } else {
                    this.entity = entity;
                }

                this.$emit('save');
                this.$message({
                    message: this.$t('adminuikit.save.success.notify'),
                    type: 'success',
                });
            } catch (e) {
                if (typeof this.onError === 'function') {
                    await this.onError();
                }

                console.error(e);
                this.$message.error({
                    message: this.$t('adminuikit.save.error.notify'),
                });
            }
        },
    },
};
