import { createNamespacedHelpers } from 'vuex';

const m = {
    SET_PROFILE_VISIBLE: 'SET_PROFILE_VISIBLE',
    SET_PROFILE_USER_ID: 'SET_PROFILE_USER_ID',
    SET_CURRENT: 'SET_CURRENT',
    WAS_CHANGED: 'WAS_CHANGED',
};

const a = {
    SHOW: 'SHOW',
    HIDE: 'HIDE',
    GET_PROFILE: 'GET_PROFILE',
    UPDATE_PROFILE: 'UPDATE_PROFILE',
};

const g = {};

export const NAMESPACE = 'profile';
export const Helper = {
    NAMESPACE: 'profile',
    MUTATIONS: m,
    ACTIONS: a,
    GETTERS: g,
    ...createNamespacedHelpers('profile'),
};

export const state = () => ({
    userId: '',
    visible: false,
    current: {
        avatar: {},
        bio: '',
        firstName: '',
        lastName: '',
        phone: '',
        email: '',
        languageCode: '',
        showContacts: Boolean,
    },
    wasChanged: false,
});

export const mutations = {
    [m.SET_PROFILE_VISIBLE](state, visible) {
        state.visible = visible;
    },

    [m.WAS_CHANGED](state, value) {
        state.wasChanged = value;
    },

    [m.SET_PROFILE_USER_ID](state, id) {
        state.userId = id;
    },

    [m.SET_CURRENT](state, profile) {
        state.current = profile;
    },
};

export const actions = {
    async [a.SHOW]({ commit, dispatch }, id) {
        await commit(m.SET_PROFILE_USER_ID, id);
        await dispatch(a.GET_PROFILE, id);
        commit(m.SET_PROFILE_VISIBLE, true);
    },

    [a.HIDE]({ commit }) {
        commit(m.SET_PROFILE_USER_ID, null);
        commit(m.SET_PROFILE_VISIBLE, false);
        commit(m.SET_CURRENT, {});
    },

    async [a.GET_PROFILE]({ commit }, id) {
        if (id) {
            let finalId = id;
            if (!finalId.match(/bo:/u)) {
                finalId = `bo:${id}`;
            }
            const profile = await this.$bo.profiles.withId(finalId).get();
            commit(m.SET_CURRENT, profile);
        }
    },

    async [a.UPDATE_PROFILE]({ commit, state }, data) {
        if (state.userId) {
            let { userId } = state;
            const arr = userId.split(':');
            if (arr.length > 1) {
                [, userId] = arr;
            }
            const profile = await this.$bo.profiles.withId(`bo:${userId}`).update(data);
            commit(m.SET_CURRENT, profile);
            commit(m.WAS_CHANGED, true);
        }
    },
};

export const getters = {
};
